# Generated by Django 2.0.1 on 2018-01-06 21:48

import django_crypto_fields.fields.encrypted_char_field
import django_crypto_fields.fields.encrypted_text_field
import edc_model.validators.phone
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("edc_locator", "0003_auto_20180103_1351")]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectlocator",
            name="subject_work_cell",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_model.validators.phone.cell_number],
                verbose_name="Work contact cell number",
            ),
        ),
        migrations.AddField(
            model_name="subjectlocator",
            name="subject_work_cell",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_model.validators.phone.cell_number],
                verbose_name="Work contact cell number",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="subject_work_phone",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_model.validators.phone.telephone_number],
                verbose_name="Work contact telephone",
            ),
        ),
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="subject_work_place",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=250,
                null=True,
                verbose_name="Name and location of work place",
            ),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="subject_work_phone",
            field=django_crypto_fields.fields.encrypted_char_field.EncryptedCharField(
                blank=True,
                help_text=" (Encryption: RSA local)",
                max_length=71,
                null=True,
                validators=[edc_model.validators.phone.telephone_number],
                verbose_name="Work contact telephone",
            ),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="subject_work_place",
            field=django_crypto_fields.fields.encrypted_text_field.EncryptedTextField(
                blank=True,
                help_text=" (Encryption: AES local)",
                max_length=250,
                null=True,
                verbose_name="Name and location of work place",
            ),
        ),
    ]



from django.db import models
from django.utils.html import format_html
from django_crypto_fields.fields import EncryptedCharField, EncryptedTextField
from edc_constants.choices import YES_NO
from edc_model.validators import cell_number, telephone_number


class SubjectIndirectContactFieldsMixin(models.Model):

    may_contact_indirectly = models.CharField(
        max_length=25,
        choices=YES_NO,
        verbose_name=format_html(
            "Has the participant given permission for study staff "
            "<b>to contact anyone else</b> for follow-up purposes during the study?"
        ),
        help_text="For example a partner, spouse, family member, neighbour ...",
    )

    indirect_contact_name = EncryptedCharField(
        verbose_name="Full names of the contact person", blank=True, null=True
    )

    indirect_contact_relation = EncryptedCharField(
        verbose_name="Relationship to participant", blank=True, null=True
    )

    indirect_contact_physical_address = EncryptedTextField(
        verbose_name="Full physical address ", max_length=500, blank=True, null=True
    )

    indirect_contact_cell = EncryptedCharField(
        verbose_name="Cell number", validators=[cell_number], blank=True, null=True
    )

    indirect_contact_cell_alt = EncryptedCharField(
        verbose_name="Cell number (alternative)",
        validators=[cell_number],
        blank=True,
        null=True,
    )

    indirect_contact_phone = EncryptedCharField(
        verbose_name="Telephone number",
        validators=[telephone_number],
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True



from django.contrib import admin


class ModelAdminLocatorMixin:
    def __init__(self, *args):
        self.get_radio_fields()
        super().__init__(*args)

    def get_fields(self, request, obj=None):
        return self.get_fields(self, request, obj=obj) + [
            "mail_address",
            "home_visit_permission",
            "physical_address",
            "may_follow_up",
            "subject_cell",
            "subject_cell_alt",
            "subject_phone",
            "subject_phone_alt",
            "may_call_work",
            "subject_work_place",
            "subject_work_phone",
            "may_contact_someone",
            "contact_name",
            "contact_rel",
            "contact_physical_address",
            "contact_cell",
            "contact_phone",
        ]

    def get_radio_fields(self):
        self.radio_fields.update(
            {
                "home_visit_permission": admin.VERTICAL,
                "may_follow_up": admin.VERTICAL,
                "may_call_work": admin.VERTICAL,
                "may_contact_someone": admin.VERTICAL,
            }
        )



# Generated by Django 2.1 on 2018-10-06 23:03

from django.db import migrations
from edc_action_item.data_fixers import fix_null_action_item_fk


def fix(apps, schema_editor):

    fix_null_action_item_fk(apps, app_label="edc_locator", models=["subjectlocator"])


class Migration(migrations.Migration):

    dependencies = [("edc_locator", "0013_auto_20181007_0054")]

    operations = [migrations.RunPython(fix)]



from django.db import models
from django.utils.html import format_html
from django_crypto_fields.fields import EncryptedCharField, EncryptedTextField
from edc_constants.choices import YES_NO
from edc_model.validators import cell_number, telephone_number


class SubjectContactFieldsMixin(models.Model):

    may_call = models.CharField(
        max_length=25,
        choices=YES_NO,
        verbose_name=format_html(
            "Has the participant given permission <b>to contacted by telephone "
            "or cell</b> by study staff for follow-up purposes during the study?"
        ),
    )

    may_visit_home = models.CharField(
        max_length=25,
        choices=YES_NO,
        verbose_name=format_html(
            "Has the participant given permission for study "
            "staff <b>to make home visits</b> for follow-up purposes?"
        ),
    )

    may_sms = models.CharField(
        max_length=25,
        choices=YES_NO,
        null=True,
        blank=False,
        verbose_name=format_html(
            "Has the participant given permission <b>to be contacted by SMS</b> "
            "by study staff for follow-up purposes during the study?"
        ),
    )

    mail_address = EncryptedTextField(
        verbose_name="Mailing address ", max_length=500, null=True, blank=True
    )

    physical_address = EncryptedTextField(
        verbose_name="Physical address with detailed description",
        max_length=500,
        blank=True,
        null=True,
        help_text="",
    )

    subject_cell = EncryptedCharField(
        verbose_name="Cell number",
        validators=[cell_number],
        blank=True,
        null=True,
        help_text="",
    )

    subject_cell_alt = EncryptedCharField(
        verbose_name="Cell number (alternate)",
        validators=[cell_number],
        blank=True,
        null=True,
    )

    subject_phone = EncryptedCharField(
        verbose_name="Telephone", validators=[telephone_number], blank=True, null=True
    )

    subject_phone_alt = EncryptedCharField(
        verbose_name="Telephone (alternate)",
        validators=[telephone_number],
        blank=True,
        null=True,
    )

    class Meta:
        abstract = True



from edc_auth.auth_objects import PII, PII_VIEW
from edc_auth.site_auths import site_auths

site_auths.update_group(
    "edc_locator.add_subjectlocator",
    "edc_locator.change_subjectlocator",
    "edc_locator.view_historicalsubjectlocator",
    "edc_locator.view_subjectlocator",
    name=PII,
)

site_auths.update_group(
    "edc_locator.view_historicalsubjectlocator",
    "edc_locator.view_subjectlocator",
    name=PII_VIEW,
)
site_auths.add_pii_model("edc_locator.subjectlocator")



from django.contrib import admin
from django_audit_fields.admin import audit_fieldset_tuple
from edc_model_admin.dashboard import ModelAdminSubjectDashboardMixin
from edc_model_admin.history import SimpleHistoryAdmin

from .admin_site import edc_locator_admin
from .fieldsets import (
    indirect_contacts_fieldset,
    subject_contacts_fieldset,
    work_contacts_fieldset,
)
from .forms import SubjectLocatorForm
from .models import SubjectLocator


@admin.register(SubjectLocator, site=edc_locator_admin)
class SubjectLocatorAdmin(ModelAdminSubjectDashboardMixin, SimpleHistoryAdmin):

    form = SubjectLocatorForm

    fieldsets = (
        (None, {"fields": ("subject_identifier",)}),
        subject_contacts_fieldset,
        work_contacts_fieldset,
        indirect_contacts_fieldset,
        audit_fieldset_tuple,
    )

    radio_fields = {
        "may_visit_home": admin.VERTICAL,
        "may_call": admin.VERTICAL,
        "may_sms": admin.VERTICAL,
        "may_call_work": admin.VERTICAL,
        "may_contact_indirectly": admin.VERTICAL,
    }

    list_filter = (
        "may_visit_home",
        "may_call",
        "may_sms",
        "may_call_work",
        "may_contact_indirectly",
    )

    list_display = (
        "subject_identifier",
        "dashboard",
        "visit_home",
        "call",
        "sms",
        "call_work",
        "contact_indirectly",
    )

    search_fields = ("subject_identifier",)



from __future__ import annotations

from typing import TYPE_CHECKING

from django.apps import apps as django_apps
from django.conf import settings
from edc_visit_schedule import site_visit_schedules

if TYPE_CHECKING:
    from .model_mixins import LocatorModelMixin


class LoctorModelError(Exception):
    pass


def get_locator_model(visit_schedule_name: str | None = None) -> str:
    """Returns the locator model name in label_lower format"""
    if visit_schedule_name:
        visit_schedule = site_visit_schedules.get_visit_schedule(visit_schedule_name)
        return visit_schedule.locator_model
    return getattr(settings, "EDC_LOCATOR_LOCATOR_MODEL", "edc_locator.subjectlocator")


def get_locator_model_cls(
    visit_schedule_name: str | None = None, locator_model: str | None = None
) -> LocatorModelMixin:
    """Returns the Locator model class.

    Uses visit_schedule_name to get the class from the visit schedule
    otherwise defaults to settings.EDC_LOCATOR_LOCATOR_MODEL.
    """
    locator_model = locator_model or get_locator_model(visit_schedule_name)
    return django_apps.get_model(locator_model)



# Generated by Django 3.2.13 on 2022-09-22 19:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("edc_locator", "0029_auto_20220826_0406"),
    ]

    operations = [
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="may_call_work",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Has the participant given permission to be contacted <b>at work</b>, by telephone or cell, by study staff for follow-up purposes during the study?",
            ),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="may_call_work",
            field=models.CharField(
                choices=[("Yes", "Yes"), ("No", "No")],
                max_length=25,
                verbose_name="Has the participant given permission to be contacted <b>at work</b>, by telephone or cell, by study staff for follow-up purposes during the study?",
            ),
        ),
    ]



# Generated by Django 2.0.3 on 2018-04-09 10:09

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [("edc_locator", "0007_auto_20180117_1819")]

    operations = [
        migrations.AddField(
            model_name="historicalsubjectlocator",
            name="parent_tracking_identifier",
            field=models.CharField(max_length=30, null=True),
        ),
        migrations.AddField(
            model_name="historicalsubjectlocator",
            name="related_tracking_identifier",
            field=models.CharField(max_length=30, null=True),
        ),
        migrations.AddField(
            model_name="subjectlocator",
            name="parent_tracking_identifier",
            field=models.CharField(max_length=30, null=True),
        ),
        migrations.AddField(
            model_name="subjectlocator",
            name="related_tracking_identifier",
            field=models.CharField(max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name="historicalsubjectlocator",
            name="tracking_identifier",
            field=models.CharField(max_length=30, null=True),
        ),
        migrations.AlterField(
            model_name="subjectlocator",
            name="tracking_identifier",
            field=models.CharField(max_length=30, null=True),
        ),
    ]



# Generated by Django 3.2.13 on 2022-08-26 01:06

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ("edc_locator", "0028_auto_20220826_0322"),
    ]

    operations = [
        migrations.RemoveField(
            model_name="historicalsubjectlocator",
            name="tracking_identifier",
        ),
        migrations.RemoveField(
            model_name="subjectlocator",
            name="tracking_identifier",
        ),
    ]



# Generated by Django 2.0.7 on 2018-07-08 17:54

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [("edc_locator", "0008_auto_20180409_1009")]

    operations = [
        migrations.AlterModelOptions(
            name="historicalsubjectlocator",
            options={
                "get_latest_by": "history_date",
                "ordering": ("-history_date", "-history_id"),
                "verbose_name": "historical Subject Locator",
            },
        ),
        migrations.RenameField(
            model_name="historicalsubjectlocator",
            old_name="parent_tracking_identifier",
            new_name="parent_reference_identifier",
        ),
        migrations.RenameField(
            model_name="historicalsubjectlocator",
            old_name="related_tracking_identifier",
            new_name="related_reference_identifier",
        ),
        migrations.RenameField(
            model_name="subjectlocator",
            old_name="parent_tracking_identifier",
            new_name="parent_reference_identifier",
        ),
        migrations.RenameField(
            model_name="subjectlocator",
            old_name="related_tracking_identifier",
            new_name="related_reference_identifier",
        ),
    ]



from unittest.case import skip

from django.contrib.messages.storage.fallback import FallbackStorage
from django.http.request import HttpRequest
from django.test import TestCase
from django.views.generic.base import ContextMixin
from edc_action_item import site_action_items
from edc_action_item.models import ActionItem
from edc_registration.models import RegisteredSubject

from ..action_items import SUBJECT_LOCATOR_ACTION
from ..view_mixins import SubjectLocatorViewMixin, SubjectLocatorViewMixinError


class DummyModelWrapper:
    def __init__(self, **kwargs):
        pass


class TestViewMixins(TestCase):
    def setUp(self):
        self.subject_identifier = "12345"
        RegisteredSubject.objects.create(subject_identifier=self.subject_identifier)

    def test_subject_locator_raises_on_bad_model(self):
        class MySubjectLocatorViewMixin(SubjectLocatorViewMixin, ContextMixin):
            subject_locator_model_wrapper_cls = DummyModelWrapper
            subject_locator_model = "blah.blahblah"

        mixin = MySubjectLocatorViewMixin()
        self.assertRaises(SubjectLocatorViewMixinError, mixin.get_context_data)

    def test_subject_locator_raisesmissing_wrapper_cls(self):
        class MySubjectLocatorViewMixin(SubjectLocatorViewMixin, ContextMixin):
            subject_locator_model = "edc_locator.subjectlocator"

        self.assertRaises(SubjectLocatorViewMixinError, MySubjectLocatorViewMixin)

    @skip("problems emulating message framework")
    def test_mixin_messages(self):
        class MySubjectLocatorViewMixin(SubjectLocatorViewMixin, ContextMixin):
            subject_locator_model_wrapper_cls = DummyModelWrapper
            subject_locator_model = "edc_locator.subjectlocator"

        mixin = MySubjectLocatorViewMixin()
        mixin.kwargs = {"subject_identifier": self.subject_identifier}
        mixin.request = HttpRequest()
        setattr(mixin.request, "session", "session")
        messages = FallbackStorage(mixin.request)
        setattr(mixin.request, "_messages", messages)
        self.assertGreater(len(mixin.request._messages._queued_messages), 0)

    def test_subject_locator_view_ok(self):
        class MySubjectLocatorViewMixin(SubjectLocatorViewMixin, ContextMixin):
            subject_locator_model_wrapper_cls = DummyModelWrapper
            subject_locator_model = "edc_locator.subjectlocator"

        mixin = MySubjectLocatorViewMixin()
        mixin.request = HttpRequest()
        setattr(mixin.request, "session", "session")
        messages = FallbackStorage(mixin.request)
        setattr(mixin.request, "_messages", messages)
        # add this manually
        mixin.kwargs = {"subject_identifier": self.subject_identifier}
        try:
            mixin.get_context_data()
        except SubjectLocatorViewMixinError as e:
            self.fail(e)

    def test_subject_locator_self_corrects_if_multiple_actionitems(self):
        class MySubjectLocatorViewMixin(SubjectLocatorViewMixin, ContextMixin):
            subject_locator_model_wrapper_cls = DummyModelWrapper
            subject_locator_model = "edc_locator.subjectlocator"

        mixin = MySubjectLocatorViewMixin()
        mixin.request = HttpRequest()
        setattr(mixin.request, "session", "session")
        messages = FallbackStorage(mixin.request)
        setattr(mixin.request, "_messages", messages)
        mixin.kwargs = {"subject_identifier": self.subject_identifier}
        try:
            mixin.get_context_data()
        except SubjectLocatorViewMixinError as e:
            self.fail(e)
        action_cls = site_action_items.get(SUBJECT_LOCATOR_ACTION)
        action_item_model_cls = action_cls.action_item_model_cls()
        action_cls(subject_identifier=self.subject_identifier)
        action_item = ActionItem.objects.get(subject_identifier=self.subject_identifier)
        self.assertEqual(action_item_model_cls.objects.all().count(), 1)
        action_item.subject_identifier = f"{self.subject_identifier}-bad"
        action_item.save()
        self.assertEqual(action_item_model_cls.objects.all().count(), 1)
        action_cls = site_action_items.get(SUBJECT_LOCATOR_ACTION)
        action_cls(subject_identifier=self.subject_identifier)
        action_item.subject_identifier = self.subject_identifier
        action_item.save()
        self.assertEqual(action_item_model_cls.objects.all().count(), 2)
        try:
            mixin.get_context_data()
        except SubjectLocatorViewMixinError as e:
            self.fail(e)
        self.assertEqual(action_item_model_cls.objects.all().count(), 1)
